@@ -153,6 +153,7 @@ class Queue(BaseJobsModel):
     def requeue_delayed_jobs(self):
         """
         Put all delayed jobs that are now ready, back in the queue waiting list
+        Return a list of failures
         """
         lock_key = self.make_key(
             self._name,
@@ -163,14 +164,14 @@ class Queue(BaseJobsModel):
 
         if connection.exists(lock_key):
             # if locked, a worker is already on it, don't wait and exit
-            return
+            return []
 
         with Lock(connection, lock_key, timeout=60):
 
             # stop here if we know we have nothing
             first_delayed_time = self.first_delayed_time
             if not first_delayed_time:
-                return None
+                return []
 
             # get when we are :)
             now_timestamp = datetime_to_score(datetime.utcnow())
@@ -178,8 +179,9 @@ class Queue(BaseJobsModel):
             # the first job will be ready later, and so the other ones too, then
             # abort
             if float(first_delayed_time) > now_timestamp:
-                return None
+                return []
 
+            failures = []
             while True:
                 # get the first entry
                 first_entry = self.first_delayed
@@ -201,9 +203,14 @@ class Queue(BaseJobsModel):
                 self.delayed.zrem(job_ident)
 
                 # and add it to the waiting queue
-                job = Job.get_from_ident(job_ident)
-                job.status.hset(STATUSES.WAITING)
-                self.enqueue_job(job)
+                try:
+                    job = Job.get_from_ident(job_ident)
+                    job.status.hset(STATUSES.WAITING)
+                    self.enqueue_job(job)
+                except Exception, e:
+                    failures.append((job_ident, '%s' % e))
+
+            return failures
 
 
 class Job(BaseJobsModel):
@@ -300,34 +307,38 @@ class Job(BaseJobsModel):
             # ok we have our job, stop now
             break
 
-        # check queue_name
-        queue_name = cls._get_queue_name(queue_name)
+        try:
+            # check queue_name
+            queue_name = cls._get_queue_name(queue_name)
 
-        # if the job already exists, and we want a higher priority or move it,
-        # start by updating it
-        if not created:
-            current_priority = int(job.priority.hget() or 0)
-            # if the job has a higher priority, or don't need to be moved,
-            # don't move it
-            if not prepend and current_priority >= priority:
-                return job
+            # if the job already exists, and we want a higher priority or move it,
+            # start by updating it
+            if not created:
+                current_priority = int(job.priority.hget() or 0)
+                # if the job has a higher priority, or don't need to be moved,
+                # don't move it
+                if not prepend and current_priority >= priority:
+                    return job
 
-            # cancel it temporarily, we'll set it as waiting later
-            job.status.hset(STATUSES.CANCELED)
+                # cancel it temporarily, we'll set it as waiting later
+                job.status.hset(STATUSES.CANCELED)
 
-            # remove it from the current queue, we'll add it to the new one later
-            if queue_model is None:
-                queue_model = cls.queue_model
-            current_queue = queue_model.get_queue(queue_name, current_priority)
-            current_queue.waiting.lrem(0, job.ident)
+                # remove it from the current queue, we'll add it to the new one later
+                if queue_model is None:
+                    queue_model = cls.queue_model
+                current_queue = queue_model.get_queue(queue_name, current_priority)
+                current_queue.waiting.lrem(0, job.ident)
 
-        else:
-            job.set_fields(added=str(datetime.utcnow()), **(fields_if_new or {}))
+            else:
+                job.set_fields(added=str(datetime.utcnow()), **(fields_if_new or {}))
 
-        # add the job to the queue
-        job.enqueue_or_delay(queue_name, priority, delayed_until, prepend, queue_model)
+            # add the job to the queue
+            job.enqueue_or_delay(queue_name, priority, delayed_until, prepend, queue_model)
 
-        return job
+            return job
+        except Exception:
+            job.queued.delete()
+            raise
 
     def run(self, queue):
         """
